{% extends "base.html" %}

{% block announce %}
  <strong>Fund this project</strong> through
  <a href="https://github.com/sponsors/pawamoy"><strong>sponsorship</strong></a>
  <span class="twemoji heart pulse">
    {% include ".icons/octicons/heart-fill-16.svg" %}
  </span> &mdash;
  Follow
  <strong>@pawamoy</strong> on
  <a rel="me" href="https://fosstodon.org/@pawamoy">
    <span class="twemoji mastodon">
      {% include ".icons/fontawesome/brands/mastodon.svg" %}
    </span>
    <strong>Fosstodon</strong>
  </a>
  for updates
{% endblock %}
